#include "matrix.h"

/* R = getRot2(q)
 * Returns the rotation matrix of a given 4-dimensional quaternion q */
void mexFunction(int lhs, mxArray *plhs[], int rhs, const mxArray *prhs[])
{
    const double *q;
    double *R;
    int M;
    int N;
    
    /* Check for correct input arguments */
    if (rhs != 1)
        mexErrMsgTxt("Number of input arguments must be 1");
    M = mxGetM(prhs[0]);
    N = mxGetN(prhs[0]);
    if ( (M != 1) && (N != 1) )
        mexErrMsgTxt("Wrong dimension of input argument");
    if ( (M != 4) && (N != 4) )
        mexErrMsgTxt("Wrong dimension of input argument");
    
    plhs[0] = mxCreateDoubleMatrix(3,3,mxREAL);
    R = mxGetPr(plhs[0]);
    q = mxGetPr(prhs[0]);
    
    R[0] = 1. - 2.*(q[2]*q[2] + q[3]*q[3]);
    R[1] = 2.*q[1]*q[2] + 2.*q[0]*q[3];
    R[2] = -2.*q[0]*q[2] + 2.*q[1]*q[3];
    R[3] = 2.*(q[1]*q[2] - q[0]*q[3]);
    R[4] = 1. - 2.*(q[1]*q[1] + q[3]*q[3]);
    R[5] = 2.*(q[0]*q[1] + q[2]*q[3]);
    R[6] = 2.*(q[0]*q[2] + q[1]*q[3]);
    R[7] = 2.*(-q[0]*q[1] + q[2]*q[3]);
    R[8] = 1. - 2.*(q[1]*q[1] + q[2]*q[2]);
    
}
