#include "matrix.h"

/* R = getRot3(NP, q)
 * Get sparse rotation matrices for every block
 * Parameters:
 * NP: Number of blocks
 * q: Generalized position
 * Returns:
 * R = Generalized rotation matrix
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int NP;
    int x, y;
    int p = 0;
    int qI;
    int *Rir;
    int *Rjc;
    double *R;
    const double *q;
    
    /* Check for correct number of input arguments */
    if (nlhs != 1 || nrhs != 2)
        mexErrMsgTxt("Wrong number of input/output arguments");
    
    NP = *(mxGetPr(prhs[0]));
    
    /* Create output matrix */
    plhs[0] = mxCreateSparse(3*NP, 3*NP, 9*NP, mxREAL);
    Rir = mxGetIr(plhs[0]);
    R = mxGetPr(plhs[0]);
    Rjc = mxGetJc(plhs[0]);
    
    /* Get input vector */
    q = mxGetPr(prhs[1]);
    
    /* Calculate R for each block p */
    for (p = 0; p < NP; p++)
    {
        qI = 7*p+3;
        x = 9*p;
        y = 3*p;
        
        R[x+0] = 1. - 2.*(q[qI+2]*q[qI+2] + q[qI+3]*q[qI+3]);
        R[x+1] = 2.*(q[qI+1]*q[qI+2] + q[qI]*q[qI+3]);
        R[x+2] = 2.*(-q[qI]*q[qI+2] + q[qI+1]*q[qI+3]);
        R[x+3] = 2.*(q[qI+1]*q[qI+2] - q[qI]*q[qI+3]);
        R[x+4] = 1. - 2.*(q[qI+1]*q[qI+1] + q[qI+3]*q[qI+3]);
        R[x+5] = 2.*(q[qI]*q[qI+1] + q[qI+2]*q[qI+3]);
        R[x+6] = 2.*(q[qI]*q[qI+2] + q[qI+1]*q[qI+3]);
        R[x+7] = 2.*(-q[qI]*q[qI+1] + q[qI+2]*q[qI+3]);
        R[x+8] = 1. - 2.*(q[qI+1]*q[qI+1] + q[qI+2]*q[qI+2]);
        
        Rir[x] = y;
        Rir[x+1] = y+1;
        Rir[x+2] = y+2;
        Rir[x+3] = y;
        Rir[x+4] = y+1;
        Rir[x+5] = y+2;
        Rir[x+6] = y;
        Rir[x+7] = y+1;
        Rir[x+8] = y+2;
        
        Rjc[y] = x;
        Rjc[y+1] = x+3;
        Rjc[y+2] = x+6;
    }
    Rjc[3*NP] = 9*NP;
}
        
        