#include "matrix.h"

/* z = Rotate2(x, q)
 * Uses the quaternion q to rotate the vector x */
void mexFunction(int lhs, mxArray *plhs[], int rhs, mxArray *prhs[])
{
    double *x;
    double *q;
    double *z;
    double qx;
    double qq;
    double tmp2[3];
    int M;
    int N;
    
    if (rhs != 2)
        mexErrMsgTxt("Number of input arguments must be two.");
    
    /* Check dimension of input arguments */
    if (mxGetM(prhs[0])*mxGetN(prhs[0]) != 3)
        mexErrMsgTxt("Input vector x must have dimension 3");
    M = mxGetM(prhs[1]);
    N = mxGetN(prhs[1]);
    if (M != 4 && N != 4)
        mexErrMsgTxt("Input quaternion q must have dimension 4");
    if (M != 1 && N != 1)
        mexErrMsgTxt("Input quaternion q must have dimension 4");
    
    /* Calculate rotation */
    plhs[0] = mxCreateDoubleMatrix(3,1,mxREAL);
    x = mxGetPr(prhs[0]);
    q = mxGetPr(prhs[1]);
    z = mxGetPr(plhs[0]);
    
    qx = q[1]*x[0] + q[2]*x[1] + q[3]*x[2];
    qq = q[1]*q[1] + q[2]*q[2] + q[3]*q[3];
    tmp2[0] = q[2]*x[2] - q[3]*x[1];
    tmp2[1] = q[3]*x[0] - q[1]*x[2];
    tmp2[2] = q[1]*x[1] - q[2]*x[0];
    
    z[0] = 2*q[1]*qx + q[0]*q[0]*x[0] + 2*q[0]*tmp2[0] - x[0]*qq;
    z[1] = 2*q[2]*qx + q[0]*q[0]*x[1] + 2*q[0]*tmp2[1] - x[1]*qq;
    z[2] = 2*q[3]*qx + q[0]*q[0]*x[2] + 2*q[0]*tmp2[2] - x[2]*qq;
}
