#include "matrix.h"

/* [Ir, Ir_inv] = getInertia(NP, I, I_inv)
* Transform the sparse inertia matrices to full 3-dimensional matrices
* Parameters:
* NP: Number of blocks
* I: Sparse inertia matrix
* I_inv: Sparse inverse inertia matrix

* Returns:
* Ir: Full inertia matrix
* Ir_inv: Full inverse inertia matrix
*/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Input parameters */
    int NP;
    const double *I;
    const int *I_ir;
    const int *I_jc;
    const double *I_inv;
    const int *I_inv_ir;
    const int *I_inv_jc;
    /* Output parameters */
    double *Ir;
    double *Ir_inv;
    /* Others */
    int dims[3];
    int p = 0;
    int i = 0;
    int tmp1 = 0;
    
    /* Check for correct number of arguments */
    if (nlhs != 2 || nrhs != 3)
        mexErrMsgTxt("Wrong number of arguments");
    
    /* Get input pointers */
    NP = *mxGetPr(prhs[0]);
    I = mxGetPr(prhs[1]);
    I_ir = mxGetIr(prhs[1]);
    I_jc = mxGetJc(prhs[1]);
    I_inv = mxGetPr(prhs[2]);
    I_inv_ir = mxGetIr(prhs[2]);
    I_inv_jc = mxGetJc(prhs[2]);
    
    /* Create new matrices */
    dims[0] = 3;
    dims[1] = 3;
    dims[2] = NP;
    plhs[0] = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL); /* Ir */
    plhs[1] = mxCreateNumericArray(3, dims, mxDOUBLE_CLASS, mxREAL); /* Ir_inv */
    
    /* Get output pointers */
    Ir = mxGetPr(plhs[0]);
    Ir_inv = mxGetPr(plhs[1]);
    
    /* Main loop */
    for (p = 0; p < NP; p++)
    {
        /* Create Ir */
        tmp1 = 6*p;
        for (i = I_jc[3*p]; i < I_jc[3*p+1]; i++)
            Ir[tmp1+I_ir[i]] = I[i];
        for (i = I_jc[3*p+1]; i < I_jc[3*p+2]; i++)
            Ir[tmp1+I_ir[i]+3] = I[i];
        for (i = I_jc[3*p+2]; i < I_jc[3*p+3]; i++)
            Ir[tmp1+I_ir[i]+6] = I[i];
            
        /* Create Ir_inv */
        for (i = I_inv_jc[3*p]; i < I_inv_jc[3*p+1]; i++)
            Ir_inv[tmp1+I_inv_ir[i]] = I_inv[i];
        for (i = I_inv_jc[3*p+1]; i < I_inv_jc[3*p+2]; i++)
            Ir_inv[tmp1+I_inv_ir[i]+3] = I_inv[i];
        for (i = I_inv_jc[3*p+2]; i < I_inv_jc[3*p+3]; i++)
            Ir_inv[tmp1+I_inv_ir[i]+6] = I_inv[i];
    }
}
