function rec105(film, mov_name)
% rec105(film, mov_name)
% film: Set to 1 to save simulation on harddrive
% mov_name: Simulation will be saved in this avi-file. Only used if film ~=
% 0

% Constant parameters
    % Constants used for the Lennard - Jones potential:
    % F = 4*k*(12*s^12/r^13 - 6*s^6/r^7)*r`
        k = 4e-5;
        s = 1.5371/2^(1/6); % Is calculated later in code

Angle = 2*pi/3;
EqAngleTheta = 0;
EqAnglePsi = 0;

% Time initialization
NT = 2000;  %3200;
dt = 0.05;

% Plot steps
nsteps = 5;
cstep = 1;

% Initialize movie
if (film ~= 0)
    mov = avifile(mov_name);
end

% Set initial positions
x = [0 0 0];
y = [0 0 0 0];
z = [0 0 0 0];
qAng = [0 2*pi/3 4*pi/3 0];
qVec = [[0,1,0]',[0,1,0]', [0,1,0]', [1,0,0]'];

% Set initial velocities
vx = [0 0 0 0];
vy = [0 0 0 0];
vz = [0 0 0 0];
wx = [0 0 0 0];
wy = [0 0 0 0];
wz = [0 0 0 0];

NP = 10; % Number of rigid bodies
NC = 5*(NP-1)+6; % Number of constraints
clf;

% Define size for needed matrices/vectors
q = zeros(7*NP,1);
v = zeros(6*NP,1);
phi = sparse(NC,1);
lambda = zeros(NC,1);
F_ext = sparse(6*NP,1);
pEdgeNext = zeros(3*NP,1);
pEdgePrev = zeros(3*NP,1);
Ui = zeros(3*NP,1);
Uj = zeros(3*NP,1);
winding = zeros(NP,1); % Winding number
last_psi = zeros(NP,1); % Angle psi from the previous timestep

T = sparse(7*NP,6*NP);
M = sparse(6*NP,6*NP);
M_dot = sparse(6*NP,6*NP);
M_inv = sparse(6*NP,6*NP);
I_body = sparse(3*NP,3*NP);
I_body_inv = sparse(3*NP,3*NP);
R = zeros(3,3);
G = sparse(NC,6*NP);
Epsilon = sparse(NC,NC);
S = sparse(NC,NC);

% Define dimension vectors
base = [2 3 3 3];
width = [0.5 1 1 1];
height = [0.5 1 1 1];
mass = 0.1*[1 1 1 1];

% Define unit body vectors
Ui = zeros(3*NP,1);
Uj = zeros(3*NP,1);
Uk = zeros(3*NP,1);

% Define charge positions 
rb = zeros(3*NP,1); % Blue charge, global system
rr = zeros(3*NP,1); % Red charge, global system
rb_body = zeros(3*NP,1); % Blue charge, body system
rr_body = zeros(3*NP,1); % Red charge, body system

% Initialize all bodies

% Set rotation of blocks
q(7*(NP-1)+4:7*(NP-1)+7) = [cos(0); sin(0)*[1,0,0]'];
p = NP-1;
while p >= 1
    qI = 7*(p-1);
    q(qI+4:qI+7) = q(7*p+4:7*p+7); % Set to previous rotation

    % Rotate this block Angle around j
    r = [cos(Angle/2), [0 1 0]*sin(Angle/2)];
    q(qI+4:qI+7) = [r(1)*q(qI+4) - dot(r(2:4), q(qI+5:qI+7)); ...
        r(1)*q(qI+5:qI+7) + q(qI+4)*r(2:4)' + cross(r(2:4)', q(qI+5:qI+7))];
    
    % Rotate this block pi/34 (for example) around k
    r = [cos(pi/34), [0 0 1]*sin(pi/34)];
    q(qI+4:qI+7) = [r(1)*q(qI+4) - dot(r(2:4), q(qI+5:qI+7)); ...
        r(1)*q(qI+5:qI+7) + q(qI+4)*r(2:4)' + cross(r(2:4)', q(qI+5:qI+7))];
    
    p = p-1;
end

for p = 1:NP
    qI = 7*(p-1); % Current position index
    vI = 6*(p-1); % Current velocity index
    
    % Initiate position vector
    % Normalize rotational direction
%    qVec(:,p) = qVec(:,p)/norm(qVec(:,p));
%    q(qI+1:qI+3) = [x(p) y(p) z(p)]';
%    q(qI+4:qI+7) = [cos(qAng(p)/2); qVec(:,p)*sin(qAng(p)/2)];
    
    % Initiate velocity vector
    v(vI+1:vI+3) = [vx(1), vy(1), vz(1)]';
    v(vI+4:vI+6) = [wx(1), wy(1), wz(1)]';
    
    % Calculate body inertia and mass matrix
        xc = 3*(p-1);
        I_body(xc+1:xc+3,xc+1:xc+3) = 1/12*mass(1)*[width(1)^2+height(1)^2,0,0;0,base(1)^2+ ...
            height(1)^2,0;0,0,base(1)^2+width(1)^2];
        I_body_inv(xc+1:xc+3, xc+1:xc+3) = inv(I_body(xc+1:xc+3, xc+1:xc+3));
        
        % Calculate rotation matrix
        R = getRot(q(qI+4:qI+7));
        
        % Calculate (temporary) inertia tensor
        xc = 3*(p-1);
        I = R*I_body(xc+1:xc+3,xc+1:xc+3)*R';
        I_inv = R*I_body_inv(xc+1:xc+3, xc+1:xc+3)*R';
        
        % Calculate generalized mass matrix
        xc = 6*(p-1);
        M(xc+1:xc+3, xc+1:xc+3) = mass(1)*eye(3);
        M(xc+4:xc+6, xc+4:xc+6) = I;
        
        % Calculate inverse mass matrix
        M_inv(xc+1:xc+3, xc+1:xc+3) = 1/mass(1)*eye(3);
        M_inv(xc+4:xc+6, xc+4:xc+6) = I_inv;
        
        % Calculate time derivative of mass matrix
        omegaSTAR = [0, -v(vI+6), v(vI+5); v(vI+6), 0, -v(vI+4); ...
            -v(vI+5), v(vI+4), 0];
        M_dot(xc+1:xc+3, xc+1:xc+3) = 0;
        M_dot(xc+4:xc+6, xc+4:xc+6) = omegaSTAR*I + I*omegaSTAR;

    % Initiate T
    T(7*(p-1)+1:7*(p-1)+3, 6*(p-1)+1:6*(p-1)+3) = eye(3);
    T(7*(p-1)+4:7*(p-1)+7, 6*(p-1)+4:6*(p-1)+6) = [-q(qI+5), -q(qI+6), -q(qI+7); ...
        q(qI+4), q(qI+7), -q(qI+6); -q(qI+7), q(qI+4), q(qI+5); q(qI+6), -q(qI+5), q(qI+4)];
    
    % Calculate edge point
    xc = 7*(p-1)+4;
    pEdgeNext(3*(p-1)+1:3*(p-1)+3) = 2*q(xc+1:xc+3)*dot(q(xc+1:xc+3),[base(1)/2,0,0]')+...
        q(xc)*q(xc)*[base(1)/2,0,0]' + ...
        2*q(xc)*cross(q(xc+1:xc+3),[base(1)/2,0,0]') - ...
        [base(1)/2,0,0]'*dot(q(xc+1:xc+3),q(xc+1:xc+3));
    
    pEdgePrev(3*(p-1)+1:3*(p-1)+3) = 2*q(xc+1:xc+3)*dot(q(xc+1:xc+3),[-base(1)/2,0,0]')+...
        q(xc)*q(xc)*[-base(1)/2,0,0]' + ...
        2*q(xc)*cross(q(xc+1:xc+3),[-base(1)/2,0,0]') - ...
        [-base(1)/2,0,0]'*dot(q(xc+1:xc+3),q(xc+1:xc+3));
    
    % Initialize unit body vectors
    Ui(3*(p-1)+1:3*(p-1)+3) = Rotate([1 0 0], q(qI+4:qI+7));
    Uj(3*(p-1)+1:3*(p-1)+3) = Rotate([0 1 0], q(qI+4:qI+7));
    Uk(3*(p-1)+1:3*(p-1)+3) = Rotate([0 0 1], q(qI+4:qI+7));
    
    % Visualization
    vertex_matrix(:,:,p)=[0,0,0;1,0,0;1,1,0;0,1,0;0,0,1;1,0,1;1,1,1;0,1,1]-0.5;
    vertex_matrix(:,1,p)=vertex_matrix(:,1,p)*base(1);
    vertex_matrix(:,2,p)=vertex_matrix(:,2,p)*width(1);
    vertex_matrix(:,3,p)=vertex_matrix(:,3,p)*height(1);
    faces_matrix=[1,2,6,5;2,3,7,6;3,4,8,7;4,1,5,8;1,2,3,4;5,6,7,8];
    
    % Rotate vertex matrix
    clear v_m;
    xc = 7*(p-1)+4;
    for i = 1:8
        v_m(i,:) = 2*q(xc+1:xc+3)'*dot(q(xc+1:xc+3),vertex_matrix(i,:,p)) + ...
            q(xc)*q(xc)*vertex_matrix(i,:,p) + ...
            2*q(xc)*cross(q(xc+1:xc+3),vertex_matrix(i,:,p)) - ...
            vertex_matrix(i,:,p)*dot(q(xc+1:xc+3),q(xc+1:xc+3));
    end
    xc = 7*(p-1)+1;
    v_m=[v_m(:,1)+q(qI+1),v_m(:,2)+q(qI+2),v_m(:,3)+q(qI+3)];
    handle(p)=patch('Vertices',v_m,'Faces',faces_matrix,...
        'FaceVertexCData',hsv(8),'FaceColor','interp');
end
view([77 8])

% Position the blocks at correct edgepoints
p = NP-1;
% Set position of last block
q(7*(NP-1)+1:7*(NP-1)+3) = [-base(1)/2 0 0]';

while p >= 1
    % Calculate distance between edgepoints
    d = q(7*p+1:7*p+3)+pEdgePrev(3*p+1:3*p+3) - ...
        q(7*(p-1)+1:7*(p-1)+3)-pEdgeNext(3*(p-1)+1:3*(p-1)+3);
    q(7*(p-1)+1:7*(p-1)+3) = q(7*(p-1)+1:7*(p-1)+3) + d;
    p = p-1;
end

% Calculate equilibrium (starting) angle theta/psi between blocks
d_x = cross(Ui(1:3), Ui(4:6));
d_dot = dot(Ui(1:3), Ui(4:6));
dt_x = cross(Uk(1:3), Uj(4:6));
dt_dot = dot(Uk(1:3), Uj(4:6));

EqAngleTheta = atan2(norm(d_x), d_dot);
EqAnglePsi = atan2(norm(dt_x), dt_dot) - pi/2;

% Place charges (red)
for p = 1:NP
    rr_body(3*(p-1)+1:3*(p-1)+3) = [0; width(1)/2; 0];
    
    % Calculate charge position in global frame
    rr(3*(p-1)+1:3*(p-1)+3) = Rotate(rr_body(3*(p-1)+1:3*(p-1)+3), q(7*(p-1)+4:7*(p-1)+7)) + ...
        q(7*(p-1)+1:7*(p-1)+3);
end

% Place charges (blue)
for p = 1:(NP-3)
    % Calculate rr for p+3 in coordinate frame p
    rr_bodyp = rr(3*(p+2)+1:3*(p+2)+3) - q(7*(p-1)+1:7*(p-1)+3);
    rr_bodyp = Rotate(rr_bodyp, [q(7*(p-1)+4); -1*q(7*(p-1)+5:7*(p-1)+7)]);
    
    % Calculate r_h in frame p
    r_h = Uj(3*(p+2)+1:3*(p+2)+3);
    r_h = Rotate(r_h, [q(7*(p-1)+4); -1*q(7*(p-1)+5:7*(p-1)+7)]);
    
    % Calculate r_abs
    r_abs = -(width(1)/2 + rr_bodyp(2))/r_h(2);
    
    % Calculate rb_body
    rb_body(3*(p-1)+1:3*(p-1)+3) = rr_bodyp + r_abs*r_h;
    
    rb(3*(p-1)+1:3*(p-1)+3) = Rotate(rb_body(3*(p-1)+1:3*(p-1)+3), q(7*(p-1)+4:7*(p-1)+7)) + ...
        q(7*(p-1)+1:7*(p-1)+3);
end

% Place last three blue charges
for p = (NP-2):NP
    rb_body(3*(p-1)+1:3*(p-1)+3) = [base(1)/8; -width(1)/2; 0];
    
    rb(3*(p-1)+1:3*(p-1)+3) = Rotate(rb_body(3*(p-1)+1:3*(p-1)+3), q(7*(p-1)+4:7*(p-1)+7)) + ...
        q(7*(p-1)+1:7*(p-1)+3);
end
    
% Find distances betwee red and blue at p and p+3 (testing)
%for p = 1:(NP-3)
%    norm(rb(3*(p-1)+1:3*(p-1)+3)-rr(3*(p+2)+1:3*(p+2)+3))
%end

% Calculate equilibrium distance between block p and block p+3
% (used with Lennard - Jones)
s = (norm(rb(1:3)-rr(3*3+1:3*3+3)))/2^(1/6);

% Calculate last_psi vector
for p = 1:(NP-1)
    dt_x(1:3) = cross(Uk(3*(p-1)+1:3*(p-1)+3), Uj(3*p+1:3*p+3));
    dt_dot = dot(Uk(3*(p-1)+1:3*(p-1)+3), Uj(3*p+1:3*p+3));
    last_psi(p) = atan2(norm(dt_x),dt_dot) - pi/2;
end

axis([-5 5 -5 5 -5 5]);
daspect([1 1 1]);

% Initiate main time loop
for t = 1:NT
    
    % Set matrices to zero
    G = G*0;
    phi = phi*0;
    F_ext = F_ext*0;
    
    for p = 1:NP
        qI = 7*(p-1);
        vI = 6*(p-1);
        if (p ~= NP)
            
            % Set constraint phi
            phi(5*(p-1)+1:5*(p-1)+3) = q(qI+1:qI+3) + pEdgeNext(3*(p-1)+1:3*(p-1)+3) - ...
                q(7*p+1:7*p+3) - pEdgePrev(3*p+1:3*p+3);
            
            % Compute Jacobian G
            edgeNextSTAR = getSTAR(pEdgeNext(3*(p-1)+1:3*(p-1)+3));
            edgePrevSTAR = getSTAR(pEdgePrev(3*p+1:3*p+3));
            G(5*(p-1)+1:5*(p-1)+3, 6*(p-1)+1:6*(p-1)+3) = eye(3);
            G(5*(p-1)+1:5*(p-1)+3, 6*(p-1)+4:6*(p-1)+6) = -edgeNextSTAR;
            
            G(5*(p-1)+1:5*(p-1)+3, 6*p+1:6*p+3) = -eye(3);
            G(5*(p-1)+1:5*(p-1)+3, 6*p+4:6*p+6) = edgePrevSTAR;
            
            % Calculate A and B
            d_x = zeros(3,1);
            d_x(1:3) = cross(Ui(3*(p-1)+1:3*(p-1)+3), Ui(3*p+1:3*p+3));
            d_dot = dot(Ui(3*(p-1)+1:3*(p-1)+3), Ui(3*p+1:3*p+3));
            dt_x = zeros(3,1);
            dt_x(1:3) = cross(Uk(3*(p-1)+1:3*(p-1)+3), Uj(3*p+1:3*p+3));
            dt_dot = dot(Uk(3*(p-1)+1:3*(p-1)+3), Uj(3*p+1:3*p+3));
            A = zeros(1,3);
            B = zeros(1,3);
            
            % Set phi
            theta = atan2(norm(d_x),d_dot);
            psi = atan2(norm(dt_x),dt_dot) - pi/2;
            
            % Add possible winding (in a primitive way, might not work for
            % all steplengths)
            if abs(last_psi(p)-psi) > pi
                if (last_psi(p) < 0) && (psi > 0)
                    winding(p) = winding(p) - 1;
                elseif (last_psi(p) > 0) && (psi < 0)
                    winding(p) = winding(p) + 1;
                end
            end
            
            % Save a new last_psi
            last_psi(p) = psi;
            
            phi(5*(p-1)+4) = theta-EqAngleTheta;
            phi(5*(p-1)+5) = psi - EqAnglePsi + 2*pi*winding(p);
            
            A(1:3) = -1/(dot(d_x,d_x)+d_dot^2)*(norm(d_x) + d_dot^2/(norm(d_x)+eps*1))*d_x;
            B(1:3) = -1/(dot(dt_x,dt_x)+dt_dot^2)*(norm(dt_x) + dt_dot^2/(norm(dt_x)+eps*1))*dt_x;
            
            % Set last part of Jacobian
            G(5*(p-1)+4, 6*(p-1)+4:6*(p-1)+6) = A;
            G(5*(p-1)+5, 6*(p-1)+4:6*(p-1)+6) = B;
            G(5*(p-1)+4, 6*p+4:6*p+6) = -A;
            G(5*(p-1)+5, 6*p+4:6*p+6) = -B;
            
            
%             % Set external forces based on the Lennard - Jones potential
%             for p2 = (p+1):NP
%                 % Calculate unit radial vector r from blue charge on p2
%                 % pointing toward red charge on p
%                 r_unit = rr(3*(p-1)+1:3*(p-1)+3) - rb(3*(p2-1)+1:3*(p2-1)+3);
%                 r = norm(r_unit);
%                 r_unit = r_unit/r;
%                 
%                 % Calculate Lennard - Jones force on red p by blue p2
%                 F = 4*k*(12*s^12/r^13 - 6*s^6/r^7)*r_unit;
%                 F_ext(6*(p-1)+1:6*(p-1)+3) = F_ext(6*(p-1)+1:6*(p-1)+3) + F;
%                 % Force on p2 by p
%                 F_ext(6*(p2-1)+1:6*(p2-1)+3) = F_ext(6*(p2-1)+1:6*(p2-1)+3) - F;
%                 
%                 % Calculate torque on p by p2
%                 F_ext(6*(p-1)+4:6*(p-1)+6) = F_ext(6*(p-1)+4:6*(p-1)+6) + ...
%                     cross(rr_body(3*(p-1)+1:3*(p-1)+3), F);
%                 % Calculate torque on p2 by p
%                 F_ext(6*(p2-1)+4:6*(p2-1)+6) = F_ext(6*(p2-1)+4:6*(p2-1)+6) + ...
%                     cross(rb_body(3*(p2-1)+1:3*(p2-1)+3), -F);
%                 
%                 
%                 % Calculate unit vector r from red charge on p2
%                 % pointing toward blue charge on p
%                 r_unit = rb(3*(p-1)+1:3*(p-1)+3) - rr(3*(p2-1)+1:3*(p2-1)+3);
%                 r = norm(r_unit);
%                 r_unit = r_unit/r;
%                 
%                 % Calculate Lennard - Jones force on blue p by red p2
%                 F = 4*k*(12*s^12/r^13 - 6*s^6/r^7)*r_unit;
%                 F_ext(6*(p-1)+1:6*(p-1)+3) = F_ext(6*(p-1)+1:6*(p-1)+3) + F;
%                 % Force on p2 by p
%                 F_ext(6*(p2-1)+1:6*(p2-1)+3) = F_ext(6*(p2-1)+1:6*(p2-1)+3) - F;
%                 
%                 % Calculate torque on p by p2
%                 F_ext(6*(p-1)+4:6*(p-1)+6) = F_ext(6*(p-1)+4:6*(p-1)+6) + ...
%                     cross(rb_body(3*(p-1)+1:3*(p-1)+3), F);
%                 % Calculate torque on p2 by p
%                 F_ext(6*(p2-1)+4:6*(p2-1)+6) = F_ext(6*(p2-1)+4:6*(p2-1)+6) + ...
%                     cross(rr_body(3*(p2-1)+1:3*(p2-1)+3), -F);
%             end
        else
            % Lock last block in origo
            phi(5*(p-1)+1:5*(p-1)+3) = q(qI+1:qI+3)+pEdgeNext(3*(p-1)+1:3*(p-1)+3);
            phi(5*(p-1)+4) = Ui(3*(p-1)+2);
            phi(5*(p-1)+5) = Ui(3*(p-1)+3);
            phi(5*(p-1)+6) = Uk(3*(p-1)+2);
            
            % Set Jacobian of last block
            edgeNextSTAR = getSTAR(pEdgeNext(3*(p-1)+1:3*(p-1)+3));
            G(5*(p-1)+1:5*(p-1)+3, 6*(p-1)+1:6*(p-1)+3) = eye(3);
            G(5*(p-1)+1:5*(p-1)+3, 6*(p-1)+4:6*(p-1)+6) = -edgeNextSTAR;
            G(5*(p-1)+4, 6*(p-1)+4:6*(p-1)+6) = cross(Ui(3*(p-1)+1:3*(p-1)+3), [0,1,0]);
            G(5*(p-1)+5, 6*(p-1)+4:6*(p-1)+6) = cross(Ui(3*(p-1)+1:3*(p-1)+3), [0,0,1]);
            G(5*(p-1)+6, 6*(p-1)+4:6*(p-1)+6) = cross(Uk(3*(p-1)+1:3*(p-1)+3), [0,1,0]);
            
            % Set external forces
        end
    end
    
    % Pull the first block outward
    KAPPA=0.001; %0.01
    F_ext(2) = F_ext(2)+KAPPA*exp(-(t-NT/2)^2/(NT/4)^2);
    if (mod(t,10)==0)
        KAPPA*exp(-(t-NT/2)^2/(NT/4)^2)
    end
    
    
    % Add motion
    F_ext = F_ext - M_dot*v;
    
    % Set Epsilon
    Epsilon = Epsilon*0;
    
    EPSA=0.000000001;  % 1
    EPSB=0.000000001;  % 0.01
    EPSC=0.0000000001;
    EPSD=0.0000000001;
    for i = 1:(NP-1)
        Epsilon(5*(i-1)+1:5*(i-1)+3,5*(i-1)+1:5*(i-1)+3) = eye(3)*EPSA;
        Epsilon(5*(i-1)+4, 5*(i-1)+4) = EPSB;
        Epsilon(5*(i-1)+5, 5*(i-1)+5) = EPSC;
    end
    Epsilon(5*(NP-1)+1:5*(NP-1)+3, 5*(NP-1)+1:5*(NP-1)+3) = eye(3)*EPSA;
    Epsilon(5*(NP-1)+4, 5*(NP-1)+4) = EPSB;
    Epsilon(5*(NP-1)+5, 5*(NP-1)+5) = EPSC;
    Epsilon(5*(NP-1)+6, 5*(NP-1)+6) = EPSD;
    
    % Define Schur complement S
    S = G*M_inv*G' + 0*dt^-2*Epsilon;
    
    % Calculate lambda
    lambda = S\(-G*v - dt*G*M_inv*F_ext - dt^-1*phi)/dt;
    
    % Compute velocity
    v = M\(G'*dt*lambda + M*v + dt*F_ext);
    
    % Update positions
    q = q + dt*T*v;
    
    % Update all other body properties
    % Set matrices to zero
    M = M*0;
    M_inv = M_inv*0;
    M_dot = M_dot*0;
    T = T*0;
    
    for p = 1:NP
        qI = 7*(p-1);
        vI = 6*(p-1);
        
        % Calculate Generalized mass matrices
            % Calculate rotation matrix
            R = getRot(q(qI+4:qI+7));
            
            % Calculate Inertia matrix
            I = R*I_body(3*(p-1)+1:3*(p-1)+3, 3*(p-1)+1:3*(p-1)+3)*R';
            I_inv = R*I_body_inv(3*(p-1)+1:3*(p-1)+3, 3*(p-1)+1:3*(p-1)+3)*R';
            
            % Update M matrices
            xc = 6*(p-1);
            M(xc+1:xc+3, xc+1:xc+3) = mass(1)*eye(3);
            M(xc+4:xc+6, xc+4:xc+6) = I;
            M_inv(xc+1:xc+3, xc+1:xc+3) = 1/mass(1)*eye(3);
            M_inv(xc+4:xc+6, xc+4:xc+6) = I_inv;
            
            % Update derivatetive of M
            omegaSTAR = getSTAR(v(vI+4:vI+6));
            M_dot(xc+4:xc+6, xc+4:xc+6) = omegaSTAR*I - I*omegaSTAR;

        % Update T
        T(7*(p-1)+1:7*(p-1)+3, 6*(p-1)+1:6*(p-1)+3) = eye(3);
        T(7*(p-1)+4:7*(p-1)+7, 6*(p-1)+4:6*(p-1)+6) = [-q(qI+5), -q(qI+6), -q(qI+7);...
            q(qI+4), q(qI+7), -q(qI+6); -q(qI+7), q(qI+4), q(qI+5);...
            q(qI+6), -q(qI+5), q(qI+4)];

        % Update edgepoints
        pEdgeNext(3*(p-1)+1:3*(p-1)+3) = Rotate([base(1)/2,0,0], q(qI+4:qI+7));
        pEdgePrev(3*(p-1)+1:3*(p-1)+3) = Rotate([-base(1)/2,0,0], q(qI+4:qI+7));
        
        % Update unit body vectors
        Ui(3*(p-1)+1:3*(p-1)+3) = Rotate([1 0 0], q(qI+4:qI+7));
        Uj(3*(p-1)+1:3*(p-1)+3) = Rotate([0 1 0], q(qI+4:qI+7));
        Uk(3*(p-1)+1:3*(p-1)+3) = Rotate([0 0 1], q(qI+4:qI+7));
        
        % Update charge positions
        rr(3*(p-1)+1:3*(p-1)+3) = ...
            Rotate(rr_body(3*(p-1)+1:3*(p-1)+3), q(7*(p-1)+4:7*(p-1)+7)) + ...
            q(7*(p-1)+1:7*(p-1)+3);
        rb(3*(p-1)+1:3*(p-1)+3) = ...
            Rotate(rb_body(3*(p-1)+1:3*(p-1)+3), q(7*(p-1)+4:7*(p-1)+7)) + ...
            q(7*(p-1)+1:7*(p-1)+3);
    end
    
    % Update graphics
    if cstep == nsteps
        for p = 1:NP
            qI = 7*(p-1);
            vI = 6*(p-1);
            
            % Normalize quaternion
            qNorm = sqrt(q(qI+4)*q(qI+4) + dot(q(qI+5:qI+7),q(qI+5:qI+7)));
            q(qI+4:qI+7) = q(qI+4:qI+7)/qNorm;
            
            % Rotate vertex matrix
            clear v_m;
            for i = 1:8
                v_m(i,:) = Rotate(vertex_matrix(i,:,p), q(qI+4:qI+7))';
            end
            v_m=[v_m(:,1)+q(qI+1),v_m(:,2)+q(qI+2),v_m(:,3)+q(qI+3)];
            set(handle(p),'Vertices',v_m);
        end
        drawnow;
        cstep = 1;
        
        % Film
        if (film ~= 0)
            frame = getframe;
            mov = addframe(mov, frame);
        end
    else
        cstep = cstep+1;
    end
end

if (film ~= 0)
    mov = close(mov);
end